import { __decorate, __param } from '../../../../../../../external/tslib/tslib.es6.js';
import { IThemeService } from 'vscode/vscode/vs/platform/theme/common/themeService';
import { IKeybindingService } from 'vscode/vscode/vs/platform/keybinding/common/keybinding';
import { IContextMenuService } from 'vscode/vscode/vs/platform/contextview/browser/contextView';
import { IConfigurationService } from 'vscode/vscode/vs/platform/configuration/common/configuration';
import { RawContextKey, IContextKeyService } from 'vscode/vscode/vs/platform/contextkey/common/contextkey';
import { localize2WithPath } from 'vscode/vscode/vs/nls';
import { IDebugService } from 'vscode/vscode/vs/workbench/contrib/debug/common/debug';
import { IEditorService } from 'vscode/vscode/vs/workbench/services/editor/common/editorService';
import { ViewPane } from 'vscode/vscode/vs/workbench/browser/parts/views/viewPane';
import { IInstantiationService } from 'vscode/vscode/vs/platform/instantiation/common/instantiation';
import { IViewDescriptorService } from 'vscode/vscode/vs/workbench/common/views';
import { IOpenerService } from 'vscode/vscode/vs/platform/opener/common/opener';
import { isCodeEditor } from 'vscode/vscode/vs/editor/browser/editorBrowser';
import { IStorageService } from 'vscode/vscode/vs/platform/storage/common/storage';
import { ITelemetryService } from 'vscode/vscode/vs/platform/telemetry/common/telemetry';
import { DisposableStore } from 'vscode/vscode/vs/base/common/lifecycle';
import { DEBUG_START_COMMAND_ID } from './debugCommands.js';

const debugStartLanguageKey = 'debugStartLanguage';
const CONTEXT_DEBUG_START_LANGUAGE = ( new RawContextKey(debugStartLanguageKey, undefined));
const CONTEXT_DEBUGGER_INTERESTED_IN_ACTIVE_EDITOR = ( new RawContextKey('debuggerInterestedInActiveEditor', false));
let WelcomeView = class WelcomeView extends ViewPane {
    static { this.ID = 'workbench.debug.welcome'; }
    static { this.LABEL = ( localize2WithPath('vs/workbench/contrib/debug/browser/welcomeView', 'run', "Run")); }
    constructor(options, themeService, keybindingService, contextMenuService, configurationService, contextKeyService, debugService, editorService, instantiationService, viewDescriptorService, openerService, storageSevice, telemetryService) {
        super(options, keybindingService, contextMenuService, configurationService, contextKeyService, viewDescriptorService, instantiationService, openerService, themeService, telemetryService);
        this.debugService = debugService;
        this.editorService = editorService;
        this.debugStartLanguageContext = CONTEXT_DEBUG_START_LANGUAGE.bindTo(contextKeyService);
        this.debuggerInterestedContext = CONTEXT_DEBUGGER_INTERESTED_IN_ACTIVE_EDITOR.bindTo(contextKeyService);
        const lastSetLanguage = storageSevice.get(debugStartLanguageKey, 1 );
        this.debugStartLanguageContext.set(lastSetLanguage);
        const setContextKey = () => {
            const editorControl = this.editorService.activeTextEditorControl;
            if (isCodeEditor(editorControl)) {
                const model = editorControl.getModel();
                const language = model ? model.getLanguageId() : undefined;
                if (language && this.debugService.getAdapterManager().someDebuggerInterestedInLanguage(language)) {
                    this.debugStartLanguageContext.set(language);
                    this.debuggerInterestedContext.set(true);
                    storageSevice.store(debugStartLanguageKey, language, 1 , 1 );
                    return;
                }
            }
            this.debuggerInterestedContext.set(false);
        };
        const disposables = ( new DisposableStore());
        this._register(disposables);
        this._register(editorService.onDidActiveEditorChange(() => {
            disposables.clear();
            const editorControl = this.editorService.activeTextEditorControl;
            if (isCodeEditor(editorControl)) {
                disposables.add(editorControl.onDidChangeModelLanguage(setContextKey));
            }
            setContextKey();
        }));
        this._register(this.debugService.getAdapterManager().onDidRegisterDebugger(setContextKey));
        this._register(this.onDidChangeBodyVisibility(visible => {
            if (visible) {
                setContextKey();
            }
        }));
        setContextKey();
        const debugKeybinding = this.keybindingService.lookupKeybinding(DEBUG_START_COMMAND_ID);
        debugKeybindingLabel = debugKeybinding ? ` (${debugKeybinding.getLabel()})` : '';
    }
    shouldShowWelcome() {
        return true;
    }
};
WelcomeView = ( __decorate([
    ( __param(1, IThemeService)),
    ( __param(2, IKeybindingService)),
    ( __param(3, IContextMenuService)),
    ( __param(4, IConfigurationService)),
    ( __param(5, IContextKeyService)),
    ( __param(6, IDebugService)),
    ( __param(7, IEditorService)),
    ( __param(8, IInstantiationService)),
    ( __param(9, IViewDescriptorService)),
    ( __param(10, IOpenerService)),
    ( __param(11, IStorageService)),
    ( __param(12, ITelemetryService))
], WelcomeView));
let debugKeybindingLabel = '';

export { WelcomeView };
